<?php

namespace Drupal\aiprompt\Form;

use Drupal\aiprompt\AIPromptSegmentPluginManager;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Routing\CurrentRouteMatch;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AIPromptSegmentForm extends FormBase {

  use DependencySerializationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * The AI Prompt Plugin manager.
   *
   * @var \Drupal\aiprompt\AIPromptSegmentPluginManager
   */
  protected $aipromptSegmentPluginManager;

  protected $entity_type;
  protected $entity;
  protected $segment_id;
  protected $segment_data;
  protected $plugin_id;
  protected $plugin_instance;

  /**
   * Constructs a new PluginForm.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, CurrentRouteMatch $currentRouteMatch, AIPromptSegmentPluginManager $aiprompt_plugin_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentRouteMatch = $currentRouteMatch;
    $this->aipromptSegmentPluginManager = $aiprompt_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('plugin.manager.aiprompt')
    );
  }

  /**
   * Returns the title for the form.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The form title.
   */
  public function getTitle() {
    $this->load();
    $label = $this->plugin_instance->getPluginDefinition()['label'];

    return $this->t('Add <em>@label</em> prompt segment', ['@label' => $label]);
  }


  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'aiprompt_plugin_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $this->load();

    $form = $this->plugin_instance->buildConfigurationForm($form, $form_state);

    $form['actions'] = [
      '#type' => 'container',
      '#weight' => 10
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary'
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $this->load();

    $this->plugin_instance->validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->load();

    $baseValues = $this->getBaseFormValues($form_state);
    $baseValues['settings'] = $this->getExtendedFormValues($form_state);
    $baseValues['plugin_id'] = $this->plugin_id;

    $this->entity->setSegment($baseValues, $this->segment_id);
    $this->entity->save();

    $form_state->setRedirect("entity.{$this->entity_type}.edit_form", [$this->entity_type => $this->entity->id()]);
  }

  /**
   * Retrieves plugin form base values.
   *
   * @param FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The base form values.
   */
  public function getBaseFormValues($form_state) {
    $data_keys = $this->plugin_instance->getBaseConfigKeys();
    return array_intersect_key($form_state->getValues(), array_flip($data_keys));
  }

  /**
   * Retrieves plugin form extended values.
   *
   * @param FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The plugin form values.
   */
  public function getExtendedFormValues($form_state) {
    $data_keys = $this->plugin_instance->getPluginConfigKeys();
    return array_intersect_key($form_state->getValues(), array_flip($data_keys));
  }

  /**
   * Loads necessary data for the form.
   *
   * This method loads the entity, segment data, plugin ID, and plugin instance.
   */
  protected function load() {
    if (!empty($this->entity)) return;

    $this->entity_type = 'aiprompt';
    $this->entity = $this->currentRouteMatch->getParameter('aiprompt');

    if (empty($this->entity)) {
      $this->entity_type = 'aiprompt_config';
      $this->entity = $this->currentRouteMatch->getParameter('aiprompt_config');
    }

    $this->segment_id = $this->currentRouteMatch->getParameter('segment_id');
    $this->plugin_id = $this->currentRouteMatch->getParameter('plugin_id');

    $this->segment_data = $this->entity->getSegment($this->segment_id);

    $this->plugin_id = !empty($this->plugin_id) ? $this->plugin_id : $this->segment_data['plugin_id'];
    $this->plugin_instance = $this->aipromptSegmentPluginManager->createInstance($this->plugin_id, $this->segment_data);
  }

}