<?php

namespace Drupal\aiprompt\Traits;

use Drupal\aiprompt\Traits\AIPromptCommons;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;

/**
 * Common form methods for prompt engineering.
 * 
 * TODO. What is the better way to inject these methods to existing forms?
 */
trait AIPromptCommonEngineeringForm {

  use AIPromptCommons;

  /**
   * Build segments form.
   */
  public function buildSegmentsForm(array $form, FormStateInterface $form_state) {

    $this->entity_type = $this->entity->getEntityTypeId();

    $element = [];

    $element['add_segment_wrapper'] = [
      '#type' => 'fieldset'
    ];
    $element['add_segment_wrapper']['plugin_select'] = [
      '#type' => 'select',
      '#title' => $this->t('Add new prompt segment'),
      '#options' => array_map(function ($definition) {
         return $definition['label'];
       }, $this->aipromptSegmentPluginManager->getDefinitions()),
      '#empty_option' => $this->t('- Select a segment type -')
    ];
    $element['add_segment_wrapper']['actions']['add_segment'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add segment'),
      '#submit' => [[$this, 'addNewSegmentSubmit']],
      '#validate' => [[$this, 'addNewSegmentValidate']]
    ];
    $element['segments_wrapper'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'segments-wrapper',
      ],
    ];
    $element['segments_wrapper']['segments_table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Segment type'),
        $this->t('Preview'),
        $this->t('Status'),
        $this->t('Operations'),
        $this->t('Weight'),
      ],
      '#empty' => $this->t('No plugins added.'),
      '#attributes' => [
        'id' => 'aiprompt-segments-table',
      ],
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'aiprompt-segments-table-row-weight',
        ],
      ],
    ];

    $segments = $this->entity->getSegments();
    $this->sortSegmentsByWeight($segments);

    foreach ($segments as $segment_id => $segment) {
      $plugin_instance = $this->aipromptSegmentPluginManager->createInstance($segment['plugin_id'], $segment);

      $item = [
        '#attributes' => ['class' => ['draggable']],
        '#weight' => $plugin_instance->getWeight(),
      ];
      $item['plugin'] = [
        '#type' => 'container'
      ];
      $item['plugin']['plugin_type'] = [
        '#type' => 'markup',
        '#markup' => $plugin_instance->getPluginDefinition()['label'],
        '#suffix' => '</br>'
      ];
      $text = nl2br($plugin_instance->getConfigurationValue('notes'));
      $item['plugin']['admin_notes'] = [
        '#type' => 'markup',
        '#markup' => $text,
        '#allowed_tags' => ['br'],
      ];

      $text = nl2br($plugin_instance->preview());
      $item['preview'] = [
        '#type' => 'markup',
        '#markup' => $text,
        '#allowed_tags' => ['br'],
      ];
      $item['status'] = [
        '#type' => 'markup',
        '#markup' => !empty($plugin_instance->getStatus()) ? $this->t('✔') : $this->t('✘'),
      ];
      $item['operations'] = [
        '#type' => 'operations',
        '#links' => [
          'edit' => [
            'title' => $this->t('Edit'),
            'url' => Url::fromRoute("{$this->entity_type}.segment.edit_form", [$this->entity_type => $this->entity->id(), 'segment_id' => $segment_id]),
          ],
          'delete' => [
            'title' => $this->t('Delete'),
            'url' => Url::fromRoute("{$this->entity_type}.segment.delete_form", [$this->entity_type => $this->entity->id(), 'segment_id' => $segment_id]),
          ],
        ],
      ];
      $item['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', ['@title' => $plugin_instance->getPluginDefinition()['label']]),
        '#title_display' => 'invisible',
        '#default_value' => $plugin_instance->getWeight(),
        '#attributes' => ['class' => ['aiprompt-segments-table-row-weight']],
      ];

      $element['segments_wrapper']['segments_table'][$segment_id] = $item;
    }

    $element['helper_actions'] = [
      '#type' => 'fieldset',
      '#title' => 'Helper actions'
    ];
    $element['helper_actions']['display_output'] = [
      '#type' => 'submit',
      '#value' => $this->t('Display output'),
      '#submit' => [[$this, 'displayOutputSubmit']]
    ];

    $form['segments_form'] = $element;
    $form['#prefix'] = '<div id="form-ajax-wrapper">';
    $form['#suffix'] = '</div>';

    return $form;
  }

  /**
   * Validate segments form.
   */
  public function addNewSegmentValidate(array &$form, FormStateInterface $form_state) {
    $selectedPluginId = $form_state->getValue('plugin_select');

    if (empty($selectedPluginId)) {
      $form_state->setErrorByName('plugin_select', $this->t('Please select a plugin first.'));
    }
  }

  /**
   * Submit callback to add new segment
   */
  public function addNewSegmentSubmit(array $form, FormStateInterface $form_state): void {
    $selectedPluginId = $form_state->getValue('plugin_select');

    $form_state->setRedirect("{$this->entity_type}.segment.add_form", [
      $this->entity_type => $this->entity->id(),
      'plugin_id' => $selectedPluginId,
    ]);
  }

  /**
   * Submit callback to display output.
   */
  public function displayOutputSubmit(array $form, FormStateInterface $form_state): void {

    $output = $this->entity->toString();

    if (class_exists('Kint') && function_exists('ksm') &&
        $this->currentUser()->hasPermission('access devel information')) {
      ksm($output);
    }
    else {
      $this->messenger()->addMessage(Markup::create(nl2br($output)));
    }
  }

}