<?php

namespace Drupal\aiprompt\Traits;

/**
 * Common methods for AIPrompt entities.
 */
trait AIPromptCommons {

  /**
   * Helper function to get sorted segments.
   */
  public function getEnabledSegmentsSortedByWeight(): array {
    $segments = $this->getSegments();

    $segments = array_filter($segments, function ($row) {
      return !empty($row['status']);
    });

    $this->sortSegmentsByWeight($segments);
    return $segments;
  }

  /**
   * Helper function to sort segments.
   */
  public function sortSegmentsByWeight(array &$segments): void {
    uasort($segments, function($a, $b) {
      return $a['weight'] - $b['weight'];
    });
  }

  /**
   * Helper function to generate segment machine name.
   */
  public function generateSegmentMachineName(string $plugin_id): string {

    $segments = $this->getSegments();

    for ($i = 1; $i < 1000; $i++) {
      $name = "{$plugin_id}_{$i}";

      if (empty($segments[$name])) {
        return $name;
      }
    }
  }

  /**
   * Gets the AI prompt plugin manager service.
   *
   * @return \Drupal\aiprompt\AIPromptSegmentPluginManager
   *   The AI prompt plugin manager service.
   */
  public function getAIPromptSegmentPluginManager() {
    if (!$this->aipromptSegmentPluginManager) {
      $this->aipromptSegmentPluginManager = \Drupal::service('plugin.manager.aiprompt');
    }

    return $this->aipromptSegmentPluginManager;
  }

  /**
   * Get complete prompt output string
   */
  public function toString(): string {
    $output = [];
    $plugin_manager = $this->getAIPromptSegmentPluginManager();

    foreach ($this->getEnabledSegmentsSortedByWeight() as $segment) {
      $plugin = $plugin_manager->createInstance($segment['plugin_id'], $segment);
      $output[] = $plugin->render();
    }
    return implode(PHP_EOL, $output);
  }

  /**
   * Preview prompt output
   *
   * @return string
   *   Preview string with added HTML line breaks
   */
  public function preview(): string {
    $output = [];
    $plugin_manager = $this->getAIPromptSegmentPluginManager();

    foreach ($this->getEnabledSegmentsSortedByWeight() as $segment) {
      $plugin = $plugin_manager->createInstance($segment['plugin_id'], $segment);
      $output[] = $plugin->preview();
    }
    $output = nl2br(implode(PHP_EOL, $output));
    return $output;
  }

  /**
   * Save weights.
   * 
   * @param array $weights
   *   Value of values from form weights element.
   */
  public function updateWeights($weights) {
    $segments = $this->getSegments();

    foreach ($weights as $machine_name => $vals) {
      $segments[$machine_name]['weight'] = $vals['weight'];
    }

    $this->setSegments($segments);
  }

}